from smllib import errors
from smllib.__version__ import __version__

# isort: split

import smllib.reader
import smllib.sml
from smllib.reader import SmlStreamReader
from smllib.sml_frame import SmlFrame
